/*
 *	clister.c - C program listing formatter.
 *
 *	Mark D. Rustad.	84/03/15.
 */


/*
 *	Include files.
 */

#include <stdio.h>
#include <stdlib.h>
#include <FCntl.h>
#include <ctype.h>
#include <Clister.h>

#include <Types.h>
#include <Quickdraw.h>

#include <CursorCtl.h>
#include <Desk.h>
#include <Files.h>
#include <Fonts.h>
#include <Memory.h>
#include <PrintTraps.h>
#include <Packages.h>
#include <Resources.h>
#include <Strings.h>
#include <ToolUtils.h>

_TITLE("clister.c - C program listing formatter.")
_SPACE("4,40,Clister")
/*
 *	Clister is an MPW tool that may be used to print C programs with
 *	line numbers, titles, subtitles and page numbering.  A companion
 *	file, clister.h defines dummy C macros to allow the C compiler to
 *	skip over the listing control directives that may be embedded in
 *	the file.  The listing control directives are:
 *
 *	_EJECT		Causes a page eject
 *
 *	_PAGE		Causes a page eject
 *
 *	_SPACE("n,l,s")	Output n blank lines and set sub-sub title to s.
 *			- Performs an eject if within n+l lines of the end
 *			  of the page.
 *			- Does not output blank lines if at top of page.
 *			- l and s may be omitted.
 *
 *	_TITLE("t")	Sets main title to t.
 *
 *	_SUBTITLE("t")	Sets subtitle to t and causes an eject.
 *
 *
 *	Clister will scan the first 100 lines of the file looking for the
 *	first _TITLE directive in order to set the title before printing begins.
 *
 *	The command line for clister takes the following form:
 *
 *	clister [options] [files]
 *
 *	-b			Print border around page
 *	-f font		Font to print text with (default Courier)
 *	-h			Print page headers
 *	-hf font	Font to print headers with (default Courier)
 *	-hs	size	Point size of header font
 *	-ls	pct		Line-spacing percentage (100 = normal spacing)
 *	-n 			Print line numbers
 *	-p			Prevents clister from marking the listing as proprietary
 *	-ps			Put up page setup dialog (page setup is then remembered)
 *	-r			Prints pages in reverse order
 *	-s size		Point size of text font
 *	-t			Uses the argument following -t as an initial title
 *
 *	The output is always written to the current printer.
 */

_SPACE("4,20,definitions")
/*
 *	Include global definitions.
 */

#define	GLOBAL
#include "glob.h"
#undef	GLOBAL


_SPACE("4,10,main")
/*
 *	main - Initialize and execute main processing loop.
 *
 *	Calls:	DisposHandle, exit,
 *		fclose, fopen, InitGraf, MoreMasters, NewHandle, phelp,
 *		selectf, sprintf, strncpy
 */

main(argc, argv)
int	argc;
char	*argv[];
{
	short	i;
	short	flag;
	void	phelp(), selectf();

	InitGraf(&qd.thePort);
	MoreMasters();
	MoreMasters();
	InitCursorCtl(0);

	/*	Allocate memory for file information.	*/

	i = argc - 1;
	if (i < 1)
		i = 1;
	Finfo = (struct finfo *)malloc(sizeof(struct finfo) * i);
	if (Finfo == 0)
	{
		fprintf(stderr, "Out of memory.\n");
		exit(1);
	}

	Nullstr = (char **)NewHandle(2);
	**Nullstr = '\0';
	HFSize = 10;
	TFSize = 7;
	POpt = 1;
	LSOpt = 100;

	GetFNum("\pCourier", &flag);
	if (flag == 0)
	{
		GetFNum("\pMonaco", &flag);
	}
	TFNum = HFNum = flag;

	if (argc < 2)			/* If no arguments, give help */
	{
		phelp(argv[0]);
		exit(0);
	}

	flag = 1;
	for (i = 1; i < argc; ++i)
	{
		if (flag && argv[i][0] == '-')
		{
			char	*p;

			c2pstr(p = argv[i] + 1);
			UprString(p, false);
			p2cstr(p);
			if (p[0] == 0)					/* If lone "-" as option	*/
				flag = 0;
			else if (strcmp(p, "B") == 0)	/* If "b" option	*/
				BOpt = 1;
			else if (strcmp(p, "F") == 0)	/* If "f" option	*/
			{
				if (++i < argc)
				{
					short	fnum;

					getfnum(argv[i], &fnum);
					TFNum = fnum;
				}
				else
				{
					fprintf(stderr, "Error in -f specification\n");
					exit(1);
				}
			}
			else if (strcmp(p, "H") == 0)	/* If "h" option	*/
				HOpt = 1;
			else if (strcmp(p, "HF") == 0)	/* If "hf" option	*/
			{
				if (++i < argc)
				{
					short	fnum;

					getfnum(argv[i], &fnum);
					HFNum = fnum;
				}
				else
				{
					fprintf(stderr, "Error in -hf specification\n");
					exit(1);
				}
			}
			else if (strcmp(p, "HS") == 0)	/* If "hs" option	*/
			{
				long	temp;

				if (++i < argc && sscanf(argv[i], "%ld", &temp) && temp != 0)
				{
					HFSize = temp;
				}
				else
				{
					fprintf(stderr, "Error in -hs option specification\n");
					exit(1);
				}
			}
			else if (strcmp(p, "N") == 0)	/* If "n" option	*/
				NOpt = 1;
			else if (strcmp(p, "P") == 0)	/* If "p" option	*/
				POpt = 0;
			else if (strcmp(p, "R") == 0)	/* If "r" option	*/
				ROpt = 1;
			else if (strcmp(p, "S") == 0)	/* If "s" option	*/
			{
				long	temp;

				if (++i < argc && sscanf(argv[i], "%ld", &temp) && temp != 0)
				{
					TFSize = temp;
				}
				else
				{
					fprintf(stderr, "Error in -s option specification\n");
					exit(1);
				}
			}
			else if (strcmp(p, "T") == 0)	/* If "t" option	*/
			{
				if (++i < argc)
					strncpy(Init_title, argv[i], TITLESIZE);
				else
				{
					fprintf(stderr, "Title missing after -t option\n");
					exit(1);
				}
			}
			else if (strcmp(p, "PS") == 0)	/* If "ps" option	*/
			{
				PSOpt = 1;
			}
			else if (strcmp(p, "LS") == 0)	/* If "ls" option	*/
			{
				long	temp;

				if (++i < argc && sscanf(argv[i], "%ld", &temp) && temp > 50)
				{
					LSOpt = temp;
				}
				else
				{
					fprintf(stderr, "Error in -ls option specification\n");
					exit(1);
				}
			}
			else
			{
				/* If unknown option	*/

				fprintf(stderr, "Unknown option - %s\n", argv[i] + 1);
				phelp(argv[0]);
				exit(1);
			}
		}
		else						/* Collect file name	*/
		{
			selectf(argv[i]);
		}
	}

	/* Get proprietary marking string.	*/

	Prop_str = (char**)GetResource('STR ', 400);
	if (Prop_str == NULL)
		Prop_str = Nullstr;			/* Set handle to null string	*/

	if (Nfiles > 0)
		print_job();

	exit(0);
}

_SUBTITLE("Subroutines.")
_SPACE("4,15,collect")
/*
 *	collect - Extract a parameter enclosed in '(' and '"' from the source
 *			string and return it, less '"', to the destination string.
 *
 *	Inputs:
 *		dest	- (param) Pointer to destination string.
 *		source	- (param) Pointer to source string.
 *
 *	Outputs:
 *		The extracted parameter is placed in the area pointed to by dest.
 *
 *	Calls:	isspace
 */

collect(dest, source)
register char *dest;			/* Pointer to destination area	*/
register char *source;			/* Pointer to source area		*/
{
	register	i = 0;

	while (isspace(*source))		/* Skip whitespace	*/
		++source;

	if (*source == '(')
	{
		++source;

		while (isspace(*source))	/* Skip whitespace	*/
			++source;

		if (*source++ == '"')
			while (*source != '\0' && *source != '"' && i++ < TITLESIZE)
				*dest++ = *source++;
	}
	
	*dest = '\0';		/* Put terminator on destination string	*/
}

_SPACE("4,15,expand_tabs")
/*
 *	expand_tabs - Expand tabs in line.
 *
 *	Inputs:
 *		co	- (param) Pointer to destination string.
 *		ci	- (param) Pointer to source string.
 *		ts	- (param) Tab stop setting.
 *
 *	Outputs:
 *		Returns pointer to next location in destination string.
 */

char	*expand_tabs(co, ci, ts)
register char	*co;	/* Pointer to destination string	*/
register char	*ci;	/* Pointer to source string			*/
int	ts;					/* Tab stop setting					*/
{
	register i, j;

	i = 0;
	while (*co = *ci++)
	{
		if (*co == '\n')
		{
			*co = '\0';
			continue;
		}
		else if (*co == '\t')
		{
			for (j = ts - (i % ts); j > 0; --j, ++i)
				*co++ = ' ';
			continue;
		}
//		else if (*co == '\f')	/* Replace form feeds with spaces	*/
//		{
//			*co = ' ';
//			continue;
//		}
		++co, ++i;
	}

	return co;
}

_SPACE("4,10,gettime")
/*
 *	gettime - Get time converted to ASCII.
 *
 *	Outputs:	Returns a pointer to the date/time string.
 *
 *	Calls:	IUDateString, IUTimeString, p2cstr, ReadDateTime
 */

char *gettime()
{
	int	len;				/* Holds string length		*/
	unsigned long	clock;	/* Holds binary time		*/
	static char	str[40];	/* Holds date/time string	*/

	ReadDateTime(&clock);
	IUDateString(clock, shortDate, str);
	len = str[0];
	p2cstr(str);
	str[len++] = ' ';
	IUTimeString(clock, false, str + len);
	p2cstr(str + len);

	return str;
}

_SPACE("4,20,phelp")
/*
 *	phelp - Print help for user.
 *
 *	Inputs:
 *		namep	- Pointer to name string.
 */

static void	phelp(namep)
char	*namep;
{
	printf("%s [options] [files]\n", namep);
	printf("-b         Print border around page\n");
	printf("-f font    Font to print text with (default Courier)\n");
	printf("-h         Print page headers\n");
	printf("-hf font   Font to print headers with (default Courier)\n");
	printf("-hs size   Size of font to print headers with (default 10 point)\n");
	printf("-ls        Specify line-spacing percentage (100 = normal spacing)\n");
	printf("-n         Print line numbers\n");
	printf("-p         Do not print proprietary notice\n");
	printf("-ps        Put up page setup dialog (page setup is then remembered)\n");
	printf("-r         Print in reverse order\n");
	printf("-s size    Size of font to print text with (default 7 point)\n");
	printf("-t         Use specified title if none in file\n");
}

_SPACE("4,15,selectf")
/*
 *	selectf - Select files for printing.
 *
 *	Inputs:
 *		path	- Path to file.
 *
 *	Calls:	faccess, strncpy
 */

void selectf(path)
char	*path;					/* Path to file to select */
{
	long	ntab;
	struct finfo	*fip;
	char	*c;

	fip = Finfo + Nfiles;

	ntab = 8;
	faccess(path, F_GTABINFO, &ntab);
	fip->ntabs = ntab;

	if ((c = malloc(strlen(path) + 1)) == 0)
	{
		fprintf(stderr, "Out of memory at %d in %s\n", __LINE__, __FILE__);
		exit(2);
	}
	strcpy(c, path);
	fip->fname = c;
//	strncpy(fip->fname, path, 255);
	++Nfiles;
}

_SPACE("4,15,tscan")
/*
 *	tscan - Scan the first 100 lines of input for the first _TITLE.
 *
 *	Inputs:	Fd	- File pointer.
 *
 *	Outputs:
 *		Init_title	- Receives title string if any was found.
 *
 *	Uses:	LineBuf.
 *
 *	Calls:	collect, fgets, fseek, strncmp.
 */

void tscan()
{
	int	slcnt = 0;				/* Scanned line count	*/
	extern long fseek();

	while (fgets(LineBuf, MAXLINE, Fd) != 0 && ++slcnt < 100)
		if (LineBuf[0] == '_' && strncmp(LineBuf, "_TITLE", 6) == 0)
		{
			collect(Init_title, &LineBuf[6]);
			break;
		}

	fseek(Fd, 0L, 0);
}

/*	End of Clister.c	*/
